/*
   Copyright (C) 2003-2012, 2014-2015, 2018-2020 Free Software Foundation, Inc.
   Written by Keisuke Nishida, Roger While, Simon Sobisch, Edward Hart

   This file is part of GnuCOBOL.

   The GnuCOBOL runtime library is free software: you can redistribute it
   and/or modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.

   GnuCOBOL is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with GnuCOBOL.  If not, see <https://www.gnu.org/licenses/>.
*/


/* COB_EXCEPTION (code, tag, name, fatal) */

/* input-output exception */
COB_EXCEPTION  (0500, COB_EC_I_O,
		"EC-I-O", 0)

/* I-O status "1x" (COBOL 2014) */
COB_EXCEPTION  (0501, COB_EC_I_O_AT_END,
		"EC-I-O-AT-END", 0)

/* An end of page condition occurred */
COB_EXCEPTION  (0502, COB_EC_I_O_EOP,
		"EC-I-O-EOP", 0)

/* A page overflow condition occurred */
COB_EXCEPTION  (0503, COB_EC_I_O_EOP_OVERFLOW,
		"EC-I-O-EOP-OVERFLOW", 0)

/* I-O status "6x" (COBOL 2014) */
COB_EXCEPTION  (0504, COB_EC_I_O_FILE_SHARING,
		"EC-I-O-FILE-SHARING", 0)

/* I-O status "9x" (COBOL 2014) */
COB_EXCEPTION  (0505, COB_EC_I_O_IMP,
		"EC-I-O-IMP", 0)

/* I-O status "2x" (COBOL 2014) */
COB_EXCEPTION  (0506, COB_EC_I_O_INVALID_KEY,
		"EC-I-O-INVALID-KEY", 0)

/* The value of a LINAGE data-item is not within the required range
   (COBOL 2014). */
COB_EXCEPTION  (0507, COB_EC_I_O_LINAGE,
		"EC-I-O-LINAGE", 1)

/* I-O status "4x" (COBOL 2014) */
COB_EXCEPTION  (0508, COB_EC_I_O_LOGIC_ERROR,
		"EC-I-O-LOGIC-ERROR", 1)

/* I-O status "3x" (COBOL 2014) */
COB_EXCEPTION  (0509, COB_EC_I_O_PERMANENT_ERROR,
		"EC-I-O-PERMANENT-ERROR", 1)

/* I-O status "7x" (COBOL 202x) */
COB_EXCEPTION  (050B, COB_EC_I_O_RECORD_CONTENT,
		"EC-I-O-RECORD-CONTENT", 1)

/* I-O status "5x" (COBOL 2014) */
COB_EXCEPTION  (050A, COB_EC_I_O_RECORD_OPERATION,
		"EC-I-O-RECORD-OPERATION", 0)

/* I-O status "0x" (COBOL 202x) */
COB_EXCEPTION  (050C, COB_EC_I_O_RECORD_WARNING,
		"EC-I-O-RECORD-WARNING", 0)
